<?php

namespace Drupal\cmlmigrations\Plugin\migrate\source;

use Drupal\cmlmigrations\Utility\MigrationsSourceBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\cmlmigrations\Utility\FindVariation;
use Drupal\cmlmigrations\Utility\FindImage;

/**
 * Source for CSV.
 *
 * @MigrateSource(
 *   id = "cml_commerce_product"
 * )
 */
class CommerceProduct extends MigrationsSourceBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration) {
    $this->fetch = FALSE;
    $this->debug = FALSE;
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);
  }

  /**
   * {@inheritdoc}
   */
  public function getRows() {
    $k = 0;
    $rows = [];
    $source = FALSE;
    $images = FALSE;
    $variations = FALSE;
    $type = \Drupal::config('cmlmigrations.settings')->get('product');
    $source = \Drupal::service('cmlapi.parser_product')->parse();
    if (!empty($source['data'])) {
      $store = $this->getStore();
      if (count($source['data']) > 400) {
        // Если вариаций много - грузим сразу все.
        $variations = FindVariation::getBy1cUuid();
        $images = FindImage::getBy1cImage();
      }
      foreach ($source['data'] as $id => $row) {
        if ($k++ < 100 || !$this->uipage) {
          $product = $row['product'];
          $offers = $row['offers'];
          // Важно для миграции с мой склад, где картинки идут в отдельном файле import.xml
          // if (!isset($product['Gruppy'])) {
          //   continue;
          // }
          $status = isset($product['status']) && $product['status'] == 'Удален' ? 0 : 1;
          $rows[$id] = [
            'uuid' => $id,
            'type' => $type,
            'stores' => $store,
            'status' => $status,
            'title' => trim($product['Naimenovanie']),
            'catalog' => $product['Gruppy'][0],
            'body' => [
              'value' => $product['Opisanie'],
              'format' => 'basic_html',
            ],
          ];
          $this->hasVariations($rows, $variations, $id);
          $this->hasImage($rows, $images, $product, $id);
        }
      }
    }
    $this->debug = FALSE;
    return $rows;
  }

  /**
   * HasVariations.
   */
  private function hasVariations(&$rows, $variations, $id) {
    $result = FALSE;
    if (!$variations) {
      // Ищем вариации текущего товара.
      $variations = FindVariation::getBy1cUuid($id, FALSE);
    }
    if (isset($variations[$id])) {
      $result = $variations[$id];
      $rows[$id]['variations'] = $result;
    }
    return $result;
  }

  /**
   * HasImage.
   */
  private function hasImage(&$rows, $images, $product, $id, $field = 'Kartinka') {
    if (!empty($product[$field])) {
      foreach ($product[$field] as $key => $image) {
        if (!$images) {
          $result = FindImage::getBy1cImage($image, FALSE)[$image];
        }
        if (isset($images[$image])) {
          $result = $images[$image];
        }
        switch ($key) {
          case 0:
            $rows[$id]['field_image'] = [
              'target_id' => $result,
            ];
            break;

          default:
            $rows[$id]['field_gallery'][] = [
              'target_id' => $result,
            ];
            break;
        }
      }
    }
  }

  /**
   * GetStore.
   */
  private function getStore() {
    $sid = FALSE;
    $storage = \Drupal::service('entity_type.manager')->getStorage('commerce_store');
    if ($store = $storage->loadDefault()) {
      $sid = $store->id();
    }
    else {
      $ids = $storage->getQuery()
        ->range(0, 1)
        ->execute();
      if (!empty($ids)) {
        $sid = array_shift($ids);
        $store = $storage->load($sid);
        $storage->markAsDefault($store);
      }
    }
    return $sid;
  }

}
